@@ -1,11 +1,11 @@
-from typing import Union, List
+from typing import Union, List, Optional
 
 import tqdm
 
 import torch
 
-from ...envs.torch import Wrapper
-from ...agents.torch import Agent
+from skrl.envs.torch import Wrapper
+from skrl.agents.torch import Agent
 
 
 def generate_equally_spaced_scopes(num_envs: int, num_agents: int) -> List[int]:
@@ -30,12 +30,12 @@ def generate_equally_spaced_scopes(num_envs: int, num_agents: int) -> List[int]:
     return scopes
 
 
-class Trainer():
+class Trainer:
     def __init__(self,
                  env: Wrapper,
                  agents: Union[Agent, List[Agent]],
-                 agents_scope : List[int] = [],
-                 cfg: dict = {}) -> None:
+                 agents_scope: Optional[List[int]] = None,
+                 cfg: Optional[dict] = None) -> None:
         """Base class for trainers
 
         :param env: Environment to train on
@@ -47,14 +47,15 @@ class Trainer():
         :param cfg: Configuration dictionary (default: {})
         :type cfg: dict, optional
         """
-        self.cfg = cfg
+        self.cfg = cfg if cfg is not None else {}
         self.env = env
         self.agents = agents
-        self.agents_scope = agents_scope
+        self.agents_scope = agents_scope if agents_scope is not None else []
 
         # get configuration
-        self.timesteps = self.cfg.get('timesteps', 0)
+        self.timesteps = self.cfg.get("timesteps", 0)
         self.headless = self.cfg.get("headless", False)
+        self.disable_progressbar = self.cfg.get("disable_progressbar", False)
 
         self.initial_timestep = 0
 
@@ -135,14 +136,6 @@ class Trainer():
         """
         raise NotImplementedError
 
-    def start(self) -> None:
-        """Start training
-
-        This method is deprecated in favour of the '.train()' method
-        """
-        # TODO: remove this method in future versions
-        print("[WARNING] Trainer.start() method is deprecated in favour of the '.train()' method")
-
     def single_agent_train(self) -> None:
         """Train a single agent
 
@@ -159,19 +152,19 @@ class Trainer():
         assert self.num_agents == 1, "This method is only valid for a single agent"
 
         # reset env
-        states = self.env.reset()
+        states, infos = self.env.reset()
 
-        for timestep in tqdm.tqdm(range(self.initial_timestep, self.timesteps)):
+        for timestep in tqdm.tqdm(range(self.initial_timestep, self.timesteps), disable=self.disable_progressbar):
 
             # pre-interaction
             self.agents.pre_interaction(timestep=timestep, timesteps=self.timesteps)
 
             # compute actions
             with torch.no_grad():
-                actions, _, _ = self.agents.act(states, timestep=timestep, timesteps=self.timesteps)
+                actions = self.agents.act(states, timestep=timestep, timesteps=self.timesteps)[0]
 
             # step the environments
-            next_states, rewards, dones, infos = self.env.step(actions)
+            next_states, rewards, terminated, truncated, infos = self.env.step(actions)
 
             # render scene
             if not self.headless:
@@ -183,7 +176,8 @@ class Trainer():
                                               actions=actions,
                                               rewards=rewards,
                                               next_states=next_states,
-                                              dones=dones,
+                                              terminated=terminated,
+                                              truncated=truncated,
                                               infos=infos,
                                               timestep=timestep,
                                               timesteps=self.timesteps)
@@ -193,8 +187,8 @@ class Trainer():
 
             # reset environments
             with torch.no_grad():
-                if dones.any():
-                    states = self.env.reset()
+                if terminated.any() or truncated.any():
+                    states, infos = self.env.reset()
                 else:
                     states.copy_(next_states)
 
@@ -214,16 +208,16 @@ class Trainer():
         assert self.num_agents == 1, "This method is only valid for a single agent"
 
         # reset env
-        states = self.env.reset()
+        states, infos = self.env.reset()
 
-        for timestep in tqdm.tqdm(range(self.initial_timestep, self.timesteps)):
+        for timestep in tqdm.tqdm(range(self.initial_timestep, self.timesteps), disable=self.disable_progressbar):
 
             # compute actions
             with torch.no_grad():
-                actions, _, _ = self.agents.act(states, timestep=timestep, timesteps=self.timesteps)
+                actions = self.agents.act(states, timestep=timestep, timesteps=self.timesteps)[0]
 
             # step the environments
-            next_states, rewards, dones, infos = self.env.step(actions)
+            next_states, rewards, terminated, truncated, infos = self.env.step(actions)
 
             # render scene
             if not self.headless:
@@ -231,19 +225,20 @@ class Trainer():
 
             with torch.no_grad():
                 # write data to TensorBoard
-                super(type(self.agents), self.agents).record_transition(states=states,
-                                                                        actions=actions,
-                                                                        rewards=rewards,
-                                                                        next_states=next_states,
-                                                                        dones=dones,
-                                                                        infos=infos,
-                                                                        timestep=timestep,
-                                                                        timesteps=self.timesteps)
+                self.agents.record_transition(states=states,
+                                              actions=actions,
+                                              rewards=rewards,
+                                              next_states=next_states,
+                                              terminated=terminated,
+                                              truncated=truncated,
+                                              infos=infos,
+                                              timestep=timestep,
+                                              timesteps=self.timesteps)
                 super(type(self.agents), self.agents).post_interaction(timestep=timestep, timesteps=self.timesteps)
 
                 # reset environments
-                if dones.any():
-                    states = self.env.reset()
+                if terminated.any() or truncated.any():
+                    states, infos = self.env.reset()
                 else:
                     states.copy_(next_states)
 
